"""
.. module:: Delhivery
.. moduleauthor:: Vineet Kumar Dubey <vineetdubey@gofynd.com>
.. note::
    It contains all the core service methods required for the delhivery.
"""

from pilotes.delhivery import DelhiveryBase
from constants import DELHIVERY_CREATE_PACKAGE, DELHIVERY_CANCEL_PACKAGE, DELHIVERY_CREATE_PICKUP, \
    DELHIVERY_CREATE_PACKAGE_HEADERS, DELHIVERY_CANCEL_PACKAGE_HEADERS, DELHIVERY_BASE_URL, DELHIVERY_DEBUG_BASE_URL
from helper import DelhiveryXMLHelper


class CreateShipment(DelhiveryBase):

    """
    Create the new shipment in delhivery.
    """

    def __init__(self, user_profile):
        super(CreateShipment, self).__init__(user_profile)

    def _prepare_pre_request_data(self, data):

        """Prepare data for request.
        TODO: Serialization to be added.
        Args:
            data - The data that will be prepared before sending it to Delhivery
        :return:
            None
        """

        self.prepared_data = data
        self.url = DELHIVERY_BASE_URL + DELHIVERY_CREATE_PACKAGE.format(self.profile.api_token)
        if self.profile.debug:
            self.url = DELHIVERY_DEBUG_BASE_URL + DELHIVERY_CREATE_PACKAGE.format(self.profile.api_token)
        self.method = 'POST'
        self.headers = DELHIVERY_CREATE_PACKAGE_HEADERS
        self.logger.info("Payload received for creating package\n{}".format(self.prepared_data))

    def _prepare_response(self):
        """Prepare response that will be returned to caller.
        Args:
            None
        :return:
            None
        """

        if self.response:
            self.response = self.response.json()


class CancelShipment(DelhiveryBase):
    """Cancels the previously created shipment in delhivery.
    """

    def __init__(self, user_profile):
        super(CancelShipment, self).__init__(user_profile)

    def _prepare_pre_request_data(self, data):

        """
        Prepare data for request.
        TODO: Serialization to be added.
        Args:
            data - The data that will be prepared before sending it to Delhivery
        :return:
            None
        """

        self.prepared_data = data
        self.url = DELHIVERY_BASE_URL + DELHIVERY_CANCEL_PACKAGE
        if self.profile.debug:
            self.url = DELHIVERY_DEBUG_BASE_URL + DELHIVERY_CANCEL_PACKAGE
        self.method = 'POST'
        self.headers['Authorization'] = DELHIVERY_CANCEL_PACKAGE_HEADERS['Authorization'].format(self.profile.api_token)
        self.logger.info("Payload received to cancel package\n{}".format(self.prepared_data))

    def _prepare_response(self):
        """
        This method is overridden.
        Prepare response in json.
        Args:
            None
        :return:
            None
        """

        formatted_response = {}
        if self.response:
            xml_response = self.response.content
            formatted_response = DelhiveryXMLHelper(xml_response).parse()
            if not formatted_response.get('status'):
                formatted_response['status'] = 'False'
        self.response = formatted_response


class CreatePickup(DelhiveryBase):
    def __init__(self, user_profile):
        super(CreatePickup, self).__init__(user_profile)

    def _prepare_pre_request_data(self, data):

        """
        Prepare data for request.
        TODO: Serialization to be added.
        :return:
        """

        self.prepared_data = data
        self.url = DELHIVERY_BASE_URL + DELHIVERY_CREATE_PICKUP
        if self.profile.debug:
            self.url = DELHIVERY_DEBUG_BASE_URL + DELHIVERY_CREATE_PICKUP
        self.method = 'POST'
        self.headers = DELHIVERY_CREATE_PACKAGE_HEADERS
        self.logger.info("Payload received for creating pickup\n{}".format(self.prepared_data))

    def _prepare_response(self):
        """
        This method is overridden.
        Prepare response in json.
        Args:
            None
        :return:
            None
        """

        if self.response.ok:
            formatted_response = self.response.json()
            formatted_response['success'] = True
        else:
            formatted_response = self.response.json()
            formatted_response['success'] = False

        self.response = formatted_response



"""
.. module:: Delhivery
.. moduleauthor:: Vineet Kumar Dubey <vineetdubey@gofynd.com>
Date: 10/02/2017
File: user_profile
This file sets the user credentials  for Delhivery profile.
"""


class UserProfile(object):
    """
    Set the user credentials for API requests.
    """
    def __init__(self, client_name, api_token, debug=False):
        """Initialization method for UserProfile class, which sets
        credentials for using Delhivery API

        Args:
            client_name - Client name for delhivery.
            api_token - Token for using the delhivery API
        """
        self.client_name = client_name
        self.api_token = api_token
        self.debug = debug



"""
File: services
Author: Om Prakash <omprakash@gofynd.com>
Date: 10/02/2017

It contains all the core service classes required for the bluedart.
"""

from pilotes.bluedart import BlueDart
from pilotes.bluedart.user_profile import UserProfile
from constants import BLUEDART_BASE_URL, BLUEDART_DEBUG_BASE_URL


class CreateShipment(BlueDart):
    """
    Create the new shipment in bluedart.
    """

    def __init__(self, profile_creds):
        request_url = BLUEDART_BASE_URL + 'WayBill/WayBillGeneration.svc?wsdl'
        if profile_creds.get('debug', False):
            request_url = BLUEDART_DEBUG_BASE_URL + 'WayBill/WayBillGeneration.svc?wsdl'
        self.request = None
        self.profile = None
        self.consignee = None
        self.shipper = None
        self.services = None
        self.user_profile = UserProfile(**profile_creds)
        super(CreateShipment, self).__init__(request_url)

    def _prepare_pre_request_data(self):
        """
        Prepare pre request WSDL prefixes.
        """

        self.Request = self.client.get_element('ns2:WayBillGenerationRequest')
        self.Profile = self.client.get_element('ns4:UserProfile')
        self.Consignee = self.client.get_element("ns2:Consignee")
        self.Shipper = self.client.get_element("ns2:Shipper")
        self.Services = self.client.get_element("ns2:Services")
        self.Commodity = self.client.get_element("ns2:CommodityDetail")
        self.dimension = self.client.get_element("ns2:Dimension")
        self.profile = self._set_profile_credentials(self.user_profile)

    def _send_request(self):
        """
        Call GenerateWayBill.
        """
        response = {}
        try:
            self.request = self.Request(
                Consignee=self.consignee,
                Shipper=self.shipper,
                Services=self.services
            )
            response = self.client.service.GenerateWayBill(self.request, self.profile)
        except Exception as exc:
            response["message"] = str(exc)

        return response


class CreatePickup(BlueDart):
    """
    This Class to register pickup request for bluedart.
    It extends the Parent class BlueDart.
    """

    def __init__(self, profile_creds):
        request_url = BLUEDART_BASE_URL + "Pickup/PickupRegistrationService.svc?wsdl"
        if profile_creds.get('debug', False):
            request_url = BLUEDART_DEBUG_BASE_URL + "Pickup/PickupRegistrationService.svc?wsdl"
        self.Request = None
        self.data_request = None
        self.data_profile = None
        self.user_profile = UserProfile(**profile_creds)
        super(CreatePickup, self).__init__(request_url)

    def _prepare_pre_request_data(self):
        """
        Prepare pre request WSDL prefixes.
        :return: None
        """
        self.Request = self.client.get_element('ns2:PickupRegistrationRequest')
        self.Profile = self.client.get_element('ns6:UserProfile')
        self.data_profile = self._set_profile_credentials(self.user_profile)

    def _send_request(self):
        """
        Send request to RegisterPickup.
        :return: response from SOAP server.
        """
        return self.client.service.RegisterPickup(self.data_request, self.data_profile)


class CancelShipment(BlueDart):
    """
    Cancel a already registered shipment request in bluedart.
    It extends the Parent class BlueDart.
    """

    def __init__(self, profile_creds):
        request_url = BLUEDART_BASE_URL + "WayBill/WayBillGeneration.svc?wsdl"
        if profile_creds.get('debug', False):
            request_url = BLUEDART_DEBUG_BASE_URL + "WayBill/WayBillGeneration.svc?wsdl"
        self.Request = None
        self.request = None
        self.profile = None
        self.awb_no = ''
        self.user_profile = UserProfile(**profile_creds)
        super(CancelShipment, self).__init__(request_url)

    def _prepare_pre_request_data(self):
        """
        Prepare pre request WSDL params.
        :return: None
        """
        self.Request = self.client.get_element('ns2:AWBCancelationRequest')
        self.Profile = self.client.get_element('ns4:UserProfile')
        self.profile = self._set_profile_credentials(self.user_profile)

    def _send_request(self):
        """
        Send request to CancelWaybill.
        :return: response from SOAP server.
        """
        response = {}
        try:
            if self.awb_no:
                self.request = self.Request(AWBNo=self.awb_no)
            response = self.client.service.CancelWaybill(self.request, self.profile)

        except Exception as exc:
            response["message"] = str(exc)

        return response



"""
File: constants
Author: Om Prakash <omprakash@gofynd.com>
Date: 16/03/2017

It contains constant values for bluedart.
"""

BLUEDART_DEBUG_BASE_URL = 'http://netconnect.bluedart.com/Demo/ShippingAPI/'
BLUEDART_BASE_URL = 'https://netconnect.bluedart.com/Ver1.7/ShippingAPI/'
TEST_CREDS = {
    "api_type": "S",
    "licence_key": "ieuiowueeriooprijnxcvoiewponcx",
    "tracking_license_key": "uyq9827iojncxlkjweu32298msdhoewo",
    "login_id": "BOM06144",
    "version": "1.7",
    "customer_code": "099960",
    "debug": True
}



"""
File: bluedart_exceptions
Author: Om Prakash <omprakash@gofynd.com>
Date: 10/02/2017

It handles the exceptions in bluedart.
"""


class BluedartException(Exception):
    """
    Bluedart Exception class.
    """

    def __init__(self, code, msg):
        self.exc_code = code
        self.msg = msg

    def __unicode__(self):
        return "Error code: {}, Message: {}".format(self.exc_code, self.msg)

    def __str__(self):
        return self.__unicode__()


class ParamValidationError(BluedartException):
    """
    Exception: Missing params handling.
    """

    def __init__(self, code=-1, message=''):
        self.exc_code = code
        try:
            self.msg += 'Missing Parameters Exception: Details: {}'.format(message)
        except AttributeError:
            pass



"""
.. module:: Delhivery
.. moduleauthor:: Vineet Kumar Dubey <vineetdubey@gofynd.com>
.. note::
    This file parses the response for all Delhivery API call.

"""


class ResponseHandler(object):
    """This is a helper class which creates valid response for all
    Delhivery API calls
    """

    def __init__(self, response):
        """Initialization method for ResponseHandler base class, which sets
        raw response that will be processed here

        Args:
            response - This will be raw response generated
        """

        self.status_code = 200
        self.response = response
        self.response_attributes = response.keys()
        self.set_properties()

    def set_properties(self):
        """It sets the properties for the response
        Args:
            None
        :return:
            Formatted JSON equivalent
        """

        for attr in self.response_attributes:
            if attr in self.response:
                self.__setattr__(attr, self.response[attr])
        return self

    def json(self):
        """It converts response to JSON
        Args:
            None
        :return:
            JSON response
        """

        json_dict = {}
        for attr in self.response_attributes:
            json_dict[attr] = self.response[attr]

        return json_dict



"""
File: services
Author: Vineet Kumar Dubey <vineetdubey@gofynd.com>
Date: 04/03/2017

It contains all the core service methods required for the delhivery.
"""
import json

from pilotes.ecomm import EcommBase
from constants import ECOM_PLACE_SHIPMENT, ECOM_CANCEL_SHIPMENT, ECOM_BASE_URL, ECOM_DEBUG_BASE_URL
from helper import ECOMXMLParser


class CreateShipment(EcommBase):

    """
    Create the new shipment in Ecomm.
    """
    def __init__(self, user_profile):
        super(CreateShipment, self).__init__(user_profile)

    def _prepare_pre_request_data(self, data):

        """
        Prepare data for request.
        TODO: Serialization to be added.
        :return:
        """
        self.prepared_data["json_input"] = json.dumps([data])
        self.url = ECOM_BASE_URL + ECOM_PLACE_SHIPMENT
        if self.profile.debug:
            self.url = ECOM_DEBUG_BASE_URL + ECOM_PLACE_SHIPMENT
        self.prepared_data['username'] = self.profile.username
        self.prepared_data['password'] = self.profile.password
        self.method = 'POST'
        self.logger.info("Payload received for creating package\n{}".format(self.prepared_data))

    def _prepare_response(self):
        try:
            if self.response:
                self.response = self.response.json()
        except:
            self.response = {}


class CancelShipment(EcommBase):
    def __init__(self, user_profile):
        super(CancelShipment, self).__init__(user_profile)

    def _prepare_pre_request_data(self, awbs):

        """
        Prepare data for request.
        TODO: Serialization to be added.
        :return:
        """
        if isinstance(awbs, list):
            self.prepared_data["awbs"] = ",".join(awbs)
        elif isinstance(awbs, str):
            self.prepared_data["awbs"] = awbs

        self.url = ECOM_BASE_URL + ECOM_CANCEL_SHIPMENT
        if self.profile.debug:
            self.url = ECOM_DEBUG_BASE_URL + ECOM_CANCEL_SHIPMENT
        self.method = 'POST'
        self.logger.info("Payload received to cancel package\n{}".format(self.prepared_data))

    def _prepare_response(self):
        """
        This method is overridden.
        Prepare response in json.
        :return: None
        """
        formatted_response = {}
        try:
            if self.response and self.response.ok:
                json_response = self.response.json()
                formatted_response = json_response[0]
            else:
                formatted_response['status'] = 'False'
            self.response = formatted_response
        except:
            self.response = {}




"""
.. module:: Delhivery
.. moduleauthor:: Vineet Kumar Dubey <vineetdubey@gofynd.com>
.. note::
    This file contains the Base class which will set the client name and api token for making the delhivery API calls
"""


import logging
import requests

from user_profile import UserProfile
from constants import DELHIVERY_CREATE_PACKAGE_HEADERS
from response_handler import ResponseHandler


class DelhiveryBase(object):
    """This class contains the abstract methods that
    are needed for preparing request and response for
    making calls Delhivery.
    """

    def __init__(self, user_profile):
        """Initialization method for Delhivery base class, which sets
        credentials for using Delhivery API

        Args:
            user_profile - This will be the dictionary containing
            client name and api token.
        """
        self.prepared_data = None
        self.url = None
        self.method = 'GET'
        self.headers = {}
        self.profile = UserProfile(**user_profile)
        self.logger = logging.getLogger(__name__)
        self.response = ResponseHandler({"success": False, "ok": False, "content": ''})

    def _prepare_pre_request_data(self, data):
        """This is abstract method and have to be over ridden in subclass.
        Inside subclass it will prepare the data for request.

        Args:
            data - It contains the payload needed for the API Call
        :return: None
        """
        pass

    def _prepare_response(self):
        """This is abstract method and have to be over ridden in subclass.
        This will prepare response accordingly.

        Args:
            None

        :return: None
        """
        pass

    def send_request(self, data):
        """
        Used to send the final JSON response and to be called by subclass.
        It internally override the _send_request of subclass.

        Args:
            data - It contains the payload needed for the API Call

        :return: Response
        """
        try:
            self._prepare_pre_request_data(data)
            if self.method == 'GET':
                self.response = requests.get(self.url)
            else:
                self.response = requests.post(self.url, data=self.prepared_data, headers=self.headers, timeout=14)

        except Exception as exc:
            self.logger.exception(exc)

        self._prepare_response()
        return self.response



"""
.. module:: Delhivery
.. moduleauthor:: Vineet Kumar Dubey <vineetdubey@gofynd.com>
.. note::
    It contains XML helper parser methods required to parse the response sent by delhivery.
"""
import xml.etree.cElementTree as ET


class DelhiveryXMLHelper(object):
    """This is a helper class which parses the xml response send
    by the delhivery to JSON
    """

    def __init__(self, xml_string):
        """Initialization method for DelhiveryXMLHelper class, which
        initializes the xml_string
        """

        self.xml_string = xml_string
        self.status_dict = dict()

    def parse(self):
        """This method simply parses the initialized xml_string to
        JSON Response
        Args:
            None
        :return: JSON equivalent of the xml_string
        """

        if self.xml_string:
            root = ET.fromstring(self.xml_string)
            for child in root:
                self.status_dict[child.tag] = child.text
        return self.status_dict



"""
File: services
Author: Vineet Kumar Dubey <vineetdubey@gofynd.com>
Date: 04/03/2017

It contains XML helper parser methods required to parse the response sent by delhivery.
"""
import xml.etree.cElementTree as ET


class ECOMXMLParser(object):
    def __init__(self, xml_string):
        self.xml_string = xml_string
        self.awb_dict = dict()
        self.awb_dict['status_list'] = []
        self.status_dict = dict()

    def parse_reverse_shipment_response(self):
        root = ET.fromstring(self.xml_string)
        root_object = root.find('AIRWAYBILL-OBJECTS')
        for object_child in root_object:
            if object_child.tag == "AIRWAYBILL":
                for child in object_child:
                    self.status_dict[child.tag] = child.text
        return self.status_dict
